import json
import os
import sys
import threading
import unittest
from collections import defaultdict
from pprint import pformat
from pprint import pprint
from greent.client import GraphQL
from collections import namedtuple
from flask_testing import LiveServerTestCase

class Vocab(object):
    root_kind         = 'http://identifiers.org/doi'
    
    # MESH
    mesh              = 'http://identifiers.org/mesh'
    mesh_disease_name = 'http://identifiers.org/mesh/disease/name'
    mesh_drug_name    = 'http://identifiers.org/mesh/drug/name'
    mesh_disease_id   = 'http://identifiers.org/mesh/disease/id'
    
    # Disease
    doid_curie          = "doid"
    doid                = "http://identifiers.org/doid"
    pharos_disease_name = "http://pharos.nih.gov/identifier/disease/name"
    
    # DRUG
    c2b2r_drug_name   = "http://chem2bio2rdf.org/drugbank/resource/Generic_Name"
    
    # TARGET
    c2b2r_gene        = "http://chem2bio2rdf.org/uniprot/resource/gene"
    
    # PATHWAY
    c2b2r_pathway     = "http://chem2bio2rdf.org/kegg/resource/kegg_pathway"
        
    # Semantic equivalence

    def __init__(self):
        self.equivalence = defaultdict(lambda: [])
        self.equivalence[self.doid_curie] = [ self.doid ]
        
        # https://github.com/prefixcommons/biocontext/blob/master/registry/uber_context.jsonld
        uber_context_path = os.path.join(os.path.dirname(__file__), 'jsonld', 'uber_context.jsonld')
        with open (uber_context_path, 'r') as stream:
            self.equivalence = json.loads (stream.read ())["@context"]
            self.equivalence["MESH"] = self.equivalence ["MESH.2013"]

class NoTranslation (Exception):
    def __init__(self, message=None):
        super(NoTranslation, self).__init__(message)

class Translator(object):

    def __init__(self, core):
        self.core = core
        self.vocab = Vocab ()
        
        # Domain translation
        self.translator_router = defaultdict (lambda: defaultdict (lambda: NoTranslation ()))
        self.translator_router[Vocab.mesh_disease_name] = {
            Vocab.mesh_drug_name : lambda disease: self.core.chemotext.disease_name_to_drug_name (disease)
        }
        self.translator_router[Vocab.doid][Vocab.mesh_disease_id]             = lambda doid:    self.core.disease_ontology.doid_to_mesh (doid.upper())
        self.translator_router[Vocab.c2b2r_drug_name][Vocab.c2b2r_gene]       = lambda drug:    self.core.chembio_ks.drug_name_to_gene_symbol (drug)
        self.translator_router[Vocab.c2b2r_gene][Vocab.c2b2r_pathway]         = lambda gene:    self.core.chembio_ks.gene_symbol_to_pathway (gene)
        self.translator_router[Vocab.c2b2r_gene][Vocab.pharos_disease_name]   = lambda gene:    self.core.pharos.target_to_disease (gene)
        self.translator_router[Vocab.mesh][Vocab.root_kind]                   = lambda mesh_id: self.core.oxo.mesh_to_other (mesh_id)
        
    def resolve_id (self, an_id, domain):
        if not an_id in domain:
            candidate = an_id
            an_id = None
            for alternative in self.vocab.equivalence[candidate]:
                if alternative in domain:
                    # Postpone the problem of synonymy
                    an_id = alternative
                    logger.debug ("Selected alternative id {0} for input {1}".format (an_id, candidate))
                    break
                # Also, if all candidates turn out not to be in the domain, we could recurse to try synonyms for them
        return an_id
    
    def translate (self, thing, domainA, domainB):
        result = None
        resolvedA = self.resolve_id (domainA, self.translator_router)
        resolvedB = self.resolve_id (domainB, self.translator_router[domainA])
        if resolvedA and resolvedB:
            result = self.translator_router[resolvedA][resolvedB] (thing)
            if isinstance (result, NoTranslation):
                raise NoTranslation ("No translation implemented from domain {0} to domain {1}".format (domainA, domainB))
        return result



from SPARQLWrapper import SPARQLWrapper2, JSON

class TripleStore(object):
    """ Connect to a SPARQL endpoint and provide services for loading and executing queries."""
    def __init__(self, hostname):
        self.service =  SPARQLWrapper2 (hostname)

    #@provenance()
    def execute_query (self, query):
        """ Execute a SPARQL query.

        :param query: A SPARQL query.
        :return: Returns a JSON formatted object.
        """
        self.service.setQuery (query)
        self.service.setReturnFormat (JSON)
        return self.service.query().convert ()
    def execute_query_file (self, query_file):
        """ Execute a SPARQL query based on a file.

        :param query_file: The file containing th query.
        :return: Returns a JSON formatted object.
        """
        result = None
        with open (query_file, "r") as stream:
            query = stream.read ()
            result = self.execute_query (query)
        return result



from collections import defaultdict
import pronto
import requests
import os

class DiseaseOntology (object):
    def __init__(self):
        self.disease_ontology_data = 'doid.obo'
        self.initialized = False
    def load (self):
        if not os.path.exists (self.disease_ontology_data):
            url = "http://purl.obolibrary.org/obo/doid.obo"
            print ("Downloading disease ontology: {0}".format (url))
            r = requests.get(url, stream=True)
            with open(self.disease_ontology_data, 'wb') as f:
                for chunk in r.iter_content(chunk_size=1024): 
                    if chunk: # filter out keep-alive new chunks
                        f.write(chunk)                
        #print ("Loading disease ontology")
        self.ont = pronto.Ontology (self.disease_ontology_data)
        ''' print (ont.json) '''


        self.doid_to_mesh_map = defaultdict(lambda:[])
        for term in self.ont:
            xref = None
            if 'xref' in term.other:
                for xref in term.other['xref']:
                    if xref.startswith ("MESH:"):
                        #print ("doid: {0} -> {1}".format (term.id.upper (), xref))
                        self.doid_to_mesh_map[term.id.upper ()].append (xref)
        self.initialized = True

    def doid_to_mesh (self, doid):
        if not self.initialized:
            self.load ()
        return self.doid_to_mesh_map [doid]
    
def test ():
    do = DiseaseOntology ()
    print (do.doid_to_mesh ("DOID:2841"))



from string import Template
import json
import os
import logging
from greent.triplestore import TripleStore

class LoggingUtil(object):
    """ Logging utility controlling format and setting initial logging level """
    @staticmethod
    def init_logging (name):
        FORMAT = '%(asctime)-15s %(filename)s %(funcName)s %(levelname)s: %(message)s'
        logging.basicConfig(format=FORMAT, level=logging.INFO)
        return logging.getLogger(name)
logger = LoggingUtil.init_logging (__file__)

class ChemBioKS(object):
    """ Generic service endpoints for medical and bio-chemical data. This set
        comprises portions of chem2bio2rdf, Monarch, and CTD environmental
        exposures."""
    def __init__(self, triplestore):
        self.triplestore = triplestore
    def get_template (self, query_name):
        query = None
        fn = os.path.join(os.path.dirname(__file__), 'query',
            '{0}.sparql'.format (query_name))
        with open (fn, 'r') as stream:
            text = stream.read ()
            query = Template (text)
            logger.debug ('query template: {0}', query)
        return query
    #@provenance()
    def query_chembio (self, query):
        """ Execute and return the result of a SPARQL query. """
        return self.triplestore.execute_query (query)

    def get_exposure_conditions (self, chemicals):
        """ Identify conditions (MeSH IDs) triggered by the specified stressor
            agent ids (also MeSH IDs).

        :param chemicals: List of IDs for substances of interest.
        :type chemicals: list of MeSH IDs, eg. D052638
        """
        id_list = ' '.join (list(map (lambda d : "( mesh:{0} )".format (d),
                            chemicals)))
        text = self.get_template ("ctd_gene_expo_disease").\
            safe_substitute (chemicals=id_list)
        results = self.triplestore.execute_query (text)
        return list(map (lambda b : {
            "chemical" : b['chemical'].value,
            "gene"     : b['gene'].value,
            "pathway"  : b['kegg_pathway'].value,
            "pathName" : b['pathway_name'].value,
            "pathID"   : b['pathway_id'].value,
            "human"    : '(human)' in b['pathway_name'].value
        },
                         results.bindings))

    def get_drugs_by_condition (self, conditions):
        """ Get drugs associated with a set of conditions.

        :param conditions: Conditions to find associated drugs for.
        :type conditions: List of MeSH IDs for conditions, eg.: D001249
        """
        condition_list = ' '.join (list(map (lambda d : "( mesh:{0} )".format (d), conditions)))
        text = self.get_template ("get_drugs_by_disease").substitute (conditions=condition_list)
        results = self.triplestore.execute_query (text)
        return list(map (lambda b : b['generic_name'].value, results.bindings))

    def get_genes_pathways_by_disease (self, diseases):
        """ Get genes and pathways associated with specified conditions.

        :param diseases: List of conditions designated by MeSH ID.
        :return: Returns a list of dicts containing gene and path information.
        """
        diseaseMeshIDList = ' '.join (list(map (lambda d : "( mesh:{0} )".format (d), diseases)))
        text = self.get_template ("genes_pathways_by_disease").safe_substitute (diseaseMeshIDList=diseaseMeshIDList)
        results = self.triplestore.execute_query (text)
        return list(map (lambda b : {
            "uniprotGene" : b['uniprotGeneID'].value,
            "keggPath"    : b['keggPath'].value,
            "pathName"    : b['pathwayName'].value,
            "human"       : '(human)' in b['pathwayName'].value
        },
        results.bindings))

    
    def get_drug_gene_disease (self, disease_name, drug_name):
        """ Identify targets and diseases assocaited with a drug name.
        :param disease_name: MeSH name of a disease condition.
        :type str: String
        :param drug_name: Name of a drug.
        :type str: String
        """
        text = self.get_template ("drug_gene_disease").safe_substitute (
            diseaseName=disease_name,
            drugName=drug_name)
        results = self.triplestore.execute_query (text)
        return list(map (lambda b : {
            "uniprotSymbol" : b['uniprotSym'].value,
            "diseaseId"     : b['diseaseID'].value
        }, results.bindings))


    def drug_name_to_gene_symbol (self, drug_name):
        print ("-----------------------------------")
        result = self.query (
            input_fields = { "drugName" : drug_name },
            output_fields = [ 'uniprotSym' ],
            query_template="""
prefix db_resource:    <http://chem2bio2rdf.org/drugbank/resource/>
prefix ctd:            <http://chem2bio2rdf.org/ctd/resource/>
prefix pubchem:        <http://chem2bio2rdf.org/pubchem/resource/>
select ?uniprotSym where {
    values ( ?drugName ) { ( "$drugName" ) }
    ?ctdChemGene ctd:cid                        ?pubChemCID;
                 ctd:gene                       ?uniprotSym.
    ?ctdChemDis  ctd:cid                        ?pubChemCID;
                 ctd:diseasename                ?diseaseName.
    ?drugID      db_resource:CID                ?pubChemCID ;
  	         db_resource:Generic_Name       ?drugGenericName .
  filter regex(lcase(str(?drugGenericName)), lcase(?drugName))
}
LIMIT 200
            """)
        return result
    
    def gene_symbol_to_pathway (self, uniprot_symbol):
        uniprot_iri = "http://chem2bio2rdf.org/uniprot/resource/gene"
        if not uniprot_symbol.startswith (uniprot_iri):
            uniprot_symbol = "{0}/{1}".format (uniprot_iri, uniprot_symbol)
            print ("----------> {0}".format (uniprot_symbol))
        return self.query (
#    values ( ?ctdGene ) { ( <http://chem2bio2rdf.org/uniprot/resource/gene/$uniprotSymbol> ) }

            input_fields = { "uniprotSymbol" : uniprot_symbol },
            output_fields = [ "keggPath" ],
            query_template="""
prefix kegg:           <http://chem2bio2rdf.org/kegg/resource/> \n
prefix pharmgkb:       <http://chem2bio2rdf.org/pharmgkb/resource/> \n
prefix ctd:            <http://chem2bio2rdf.org/ctd/resource/> \n
select ?ctdGene ?uniprotID ?pathwayName ?keggPath where { \n
    values ( ?ctdGene ) { ( <$uniprotSymbol> ) } \n
    ?keggPath    kegg:protein    ?uniprotID ; kegg:Pathway_name ?pathwayName . \n
    ?pharmGene   pharmgkb:Symbol ?ctdGene ; pharmgkb:UniProt_Id ?uniprotID. \n
    ?ctdChemGene ctd:gene        ?ctdGene. \n
}
""")
    
    def query (self, query_template, output_fields, input_fields=[]):
        query_template = Template(query_template)
        query_text = query_template.safe_substitute (**input_fields)
        query_results = self.triplestore.execute_query (query_text)
        #print (query_results.bindings)
        result = set ()
        for b in query_results.bindings:
            for f in output_fields:
                result.add (b[f].value)
        return result
#        return { key : value.value for (key, value) in query_results.bindings }
        



import json
import logging
import unittest
import requests
import sys
import traceback
import requests
from string import Template
from requests.packages.urllib3.exceptions import InsecureRequestWarning
requests.packages.urllib3.disable_warnings(InsecureRequestWarning)

class LoggingUtil(object):
    """ Logging utility controlling format and setting initial logging level """
    @staticmethod
    def init_logging (name):
        level = logging.DEBUG
        FORMAT = '%(asctime)-15s %(filename)s %(funcName)s %(levelname)s: %(message)s'
        logging.basicConfig(format=FORMAT, level=level)
        return logging.getLogger(name)

logger = LoggingUtil.init_logging (__file__)

class GraphQL (object):

    ''' GraphQL client for the Green Translator. '''

#    http://localhost:5000/patients/

    def __init__(self, url="https://stars-app.renci.org/greent/graphql"):
        self.url = url

    def query (self, query):
        logger.debug ("query: %s", query)
        r = requests.post (self.url,
                           data = json.dumps (query).encode ('utf8'),
                           headers = { 'Content-Type': 'application/json' })
        r.raise_for_status ()
        return r.json ()

    # Exposures

    def get_exposure_scores (self, exposure_type, start_date, end_date, exposure_point):
        response = self.query ({
            "query" : """
query getExposureScore($exposureType:String, $startDate:String, $endDate:String, $exposurePoint:String){
  exposureScore(exposureType  : $exposureType, 
    		startDate     : $startDate, 
    		endDate       : $endDate, 
    	        exposurePoint : $exposurePoint)
  {
    value
    latitude
    longitude
  }
}
""",
            "variables" : {
                "exposureType"  : exposure_type,
                "startDate"     : start_date,
                "endDate"       : end_date,
                "exposurePoint" : exposure_point
            }
        })
        return response['data']['exposureScore']['scores']

    def get_exposure_values (self, exposure_type, start_date, end_date, exposure_point):
        response = self.query ({
            "query" : """
query getExposureValue($exposureType:String, $startDate:String, $endDate:String, $exposurePoint:String){
  exposureValue(exposureType  : $exposureType, 
    		startDate     : $startDate, 
    		endDate       : $endDate , 
    	        exposurePoint : $exposurePoint)
  {
    value
    latitude
    longitude
  }
}
""",
            "variables" : {
                "exposureType"  : exposure_type,
                "startDate"     : start_date,
                "endDate"       : end_date,
                "exposurePoint" : exposure_point
            }
        })
        return response['data']['exposureValue']

    # ChemBio API

    def get_exposure_conditions (self, chemicals):
        response = self.query ({
            "query" : """
query getExposureConditions($chem : [String] ) {
  exposureConditions(chemicals: $chem) {
    chemical
    gene
    pathway
    pathName
    pathID
    human
  }
}
""",
            "variables" : {
                "chem" : chemicals
            }
        })
        return response ['data']['exposureConditions']
    
    def get_drugs_by_condition (self, conditions):
        response = self.query ({
            "query" : """
query getDrugsByCondition($cond : [String] ) {
    drugsByCondition(conditions:$cond) {
      genericName
    }
}
            """,
            "variables" : {
                "cond" : conditions
            }
        })
        drugs = response['data']['drugsByCondition']
        return [ r['genericName'] for r in drugs ]
 
    def get_genes_pathways_by_disease (self, diseases):
        response = self.query ({
            "query" : """
query getGenesPathways($diseases : [String] ) {
    genePathsByDisease(diseases:$diseases) {
      uniprotGene
      keggPath
      pathName
      human
    }
}
""",
            "variables" : {
                "diseases" : diseases
            }
        })
        return response['data']['genePathsByDisease']

    # Clinical API

    def get_patients (self, age=None, sex=None, race=None, location=None):
        response = self.query ({
            "query" : """
                query queryPatients ($age : String, $sex : String, $race : String) {
                    patients (age: $age, sex: $sex, race: $race)
                }
            """,
            "variables" : {
                "age"  : age,
                "sex"  : sex,
                "race" : race
            }
        })
        return response['data']['patients']

    def translate (self, thing, domain_a, domain_b):
        query_text = Template ("""{ translate (thing:\"$thing\", domainA: \"$domain_a\", domainB: \"$domain_b\") { value } }""").\
           safe_substitute (thing=thing, domain_a=domain_a, domain_b=domain_b)
        result = self.query ({
            "query" : query_text
        })['data']['translate']
        return result
    
class TestGraphQLClient (unittest.TestCase):
 
    client = GraphQL ("http://localhost:5000/graphql")

    def test_get_exposure_scores (self):
        lat = "35.9131996"
        lon = "-79.0558445"
        result = self.client.get_exposure_scores (
            exposure_type = "pm25",
            start_date = "2010-01-01",
            end_date = "2010-01-07",
            exposure_point = ",".join ([ lat, lon ]))
        #print ("-----------> {}".format (response))
        #result = json.loads (text)
        #print (json.dumps (result, indent=2))
#        self.assertTrue (type (result[0]['latitude']) == float)
#        self.assertTrue (type (result[0]['longitude']) == float)
# until the new exposures service is up, this is not working
        self.assertTrue (result['latitude'] == None)
        self.assertTrue (result['longitude'] == None)
        print ("Verified exposure response structure")

    def test_get_exposure_values (self):
        lat = "35.9131996"
        lon = "-79.0558445"
        result = self.client.get_exposure_values (
            exposure_type = "pm25",
            start_date = "2010-01-01",
            end_date = "2010-01-07",
            exposure_point = ",".join ([ lat, lon ]))
        #result = json.loads (text)
        #print (json.dumps (result, indent=2))
#        self.assertTrue (type (result[0]['latitude']) == float)
#        self.assertTrue (type (result[0]['longitude']) == float)
# until the new exposures service is up, this is not working
        self.assertTrue (result['latitude'] == None)
        self.assertTrue (result['longitude'] == None)

        print ("Verified exposure response structure")

    def test_get_exposure_conditions (self):
        result = self.client.get_exposure_conditions ([ "D052638" ])
        #result = json.loads (result ['data']['exposureConditions'])
        result = sorted (result, key=lambda v : v['gene'])
        self.assertTrue (result[0]['gene'] == 'http://chem2bio2rdf.org/uniprot/resource/gene/COL1A1')

    def test_get_drugs_by_condition (self):
        result = self.client.get_drugs_by_condition ([ "d001249" ])
        self.assertTrue ("Melphalan" in result)

    def test_get_genes_pathways_by_disease (self):
        result = self.client.get_genes_pathways_by_disease ([ "d001249" ])
        result = sorted (result, key=lambda v : v['uniprotGene'])
        self.assertTrue (result[0]['uniprotGene'] == 'http://chem2bio2rdf.org/uniprot/resource/gene/ABCB1')

    def test_clinical (self):
        pass #print (self.client.get_patients ())
    
if __name__ == '__main__':
    unittest.main()




import requests
import json

class Pharos(object):
    def __init__(self, url="https://pharos.nih.gov/idg/api/v1"):
        self.url = url
    def request (self, url):
        return requests.get (url).json ()
#            url = url, 
#            headers={ "Content-Type" : "application/json" }).json ()
    def query (self, query):
        return self.request ("{0}/{1}".format (self.url, query))
    def target_to_id (self, target):
        target_map = self.query (query="targets/{0}".format (target))
        return target_map["id"]
    def target_to_disease (self, target_sym):
        target_id = self.target_to_id (target_sym)
        links = self.query (query="targets({0})/links".format (target_id))
        result = []
        for k in links:
            #print (k['id'])
            if k['kind'] == "ix.idg.models.Disease":
                for p in k['properties']:
                    #print (p)
                    if p['label'] == 'IDG Disease':
                        result.append (p['term'])
        for r in result:
            print (r)
        return result
    
        #https://pharos.nih.gov/idg/api/v1/targets(15633)/links
    def disease_map (self, disease_id):
        return self.query (query="diseases({0})".format (disesase_id))

def test ():
#    with open("pharos.txt", "r") as stream:
#        obj = json.loads (stream.read ())
#        print (json.dumps (obj, indent=2))        
    pharos = Pharos ()
    diseases = pharos.target_to_disease ("CACNA1A")
    print (diseases)



import requests
import json

class Neo4JREST(object):
    def __init__(self, url):
        self.url = url
        self.query_endpoint = "{0}/db/data/transaction".format (self.url)
        self.schema_endpoint = "{0}/db/data/schema/".format (self.url)

    def request (self, url, obj):
        return requests.post (url = self.query_endpoint,
                             data = json.dumps (obj, indent=2),
                             headers={ "Content-Type" : "application/json" }).json ()
        
    def query (self, query):
        return self.request (
            url = self.query_endpoint,
            obj = {
                "statements": [
                    {
                        "statement": query,
                        "resultDataContents": [
                            "row",
                            "graph"
                        ],
                        "includeStats": True
                }
                ]
            })

class Chemotext(Neo4JREST):
    def __init__(self, url="http://chemotext.mml.unc.edu:7474"):
        super (Chemotext, self).__init__(url)

    def term_to_term (self, A, limit=100):
        return self.query (
            query="MATCH (d:Term)-[r1]-(a:Art)-[r2]-(t:Term) WHERE d.name='%s' RETURN d, r1, a, r2, t LIMIT %s" % (A, limit))

    def disease_name_to_drug_name (self, disease, limit=100):
        result = []
        response = self.query (
            query="MATCH (d:Term {type:'Disease', name: '%s' })-[r1]-(a:Art)-[r2]-(t:Term {isDrug:true}) RETURN d, r1, a, r2, t LIMIT %s" %
            (disease, limit))
        #print (json.dumps (response, indent=2))
        for r in response['results'][0]['data']:
            result.append (r['row'][4]['name'])
        return result
    
def test ():
    chemotext = Chemotext ()
    #obj = chemotext.disease_to_drug ("Asthma")
    #print (json.dumps (obj, indent=2))
    
    obj = chemotext.term_to_term ("Asthma")
    print (json.dumps (obj, indent=2))



import datetime
import logging
import json
import pprint
import unittest
import os
from collections import defaultdict
from greent.triplestore import TripleStore
from greent.chembio import ChemBioKS
from greent.chemotext import Chemotext
#from greent.exposures import Exposures
from greent.clinical import Clinical
from greent.disease_ont import DiseaseOntology
from greent.cmaq import CMAQ
from greent.pharos import Pharos
from greent.oxo import OXO
from greent.translator import Translator

class LoggingUtil(object):
    """ Logging utility controlling format and setting initial logging level """
    @staticmethod
    def init_logging (name):
        FORMAT = '%(asctime)-15s %(filename)s %(funcName)s %(levelname)s: %(message)s'
        logging.basicConfig(format=FORMAT, level=logging.INFO)
        return logging.getLogger(name)

logger = LoggingUtil.init_logging (__file__)

class GreenT (object):

    ''' The Green Translator API - a single Python interface aggregating access mechanisms for 
    all Green Translator services. '''

    def __init__(self, config={}):
        self.config = config
        
        blaze_uri = self.get_config ('blaze_uri', 'http://stars-blazegraph.renci.org/bigdata/sparql')
        self.blazegraph = TripleStore (blaze_uri)

        self.chembio_ks = ChemBioKS (self.blazegraph)
        clinical_url = self.get_config ('clinical_url', "http://tweetsie.med.unc.edu/CLINICAL_EXPOSURE")

        self.clinical = Clinical (swagger_endpoint_url=clinical_url)
        exposures_uri = self.get_config ("exposures_uri",
                                         "https://app.swaggerhub.com/apiproxy/schema/file/mjstealey/environmental_exposures_api/0.0.1/swagger.json")
        self.exposures = CMAQ (exposures_uri)
        self.chemotext = Chemotext ()
        self.disease_ontology = DiseaseOntology ()
        self.pharos = Pharos ()
        self.oxo = OXO ()
        #self.init_translator ()
        self.translator = Translator (core=self)
        
    def get_config (self, key, default):
        result = None
        if key in self.config:
            result = self.config[key]
        if not result:
            result = default
        return result
                                  
    # Exposure API

    def get_exposure_scores (self, exposure_type, start_date, end_date, exposure_point):
        return self.exposures.get_scores (
            exposure_type = exposure_type,
            start_date    = start_date,
            end_date      = end_date,
            lat_lon       = exposure_point)

    def get_exposure_values (self, exposure_type, start_date, end_date, exposure_point):
        return self.exposures.get_values (
            exposure_type  = exposure_type,
            start_date     = start_date,
            end_date       = end_date,
            lat_lon        = exposure_point)

    # ChemBio API

    def get_exposure_conditions_json (self, chemicals):
        return json.dumps (self.get_exposure_conditions (chemicals))

    def get_exposure_conditions (self, chemicals):
        return self.chembio_ks.get_exposure_conditions (chemicals)

    def get_drugs_by_condition_json (self, conditions):
        return json.dumps (self.get_drugs_by_condition (conditions))

    def get_drugs_by_condition (self, conditions):
        return self.chembio_ks.get_drugs_by_condition (conditions)

    def get_genes_pathways_by_disease_json (self, diseases):
        return json.dumps (self.get_genes_pathways_by_disease (diseases))

    def get_genes_pathways_by_disease (self, diseases):
        return self.chembio_ks.get_genes_pathways_by_disease (diseases)

    def get_drug_gene_disease (self, disease_name, drug_name):
        return self.chembio_ks.get_drug_gene_disease (disease_name, drug_name)
    
    # Clinical API

    def get_patients (self, age=None, sex=None, race=None, location=None):
        return self.clinical.get_patients (age, sex, race, location)

    # Translator
    '''
    def init_translator (self):
        root_kind         = 'http://identifiers.org/doi'

        # MESH
        mesh              = 'http://identifiers.org/mesh'
        mesh_disease_name = 'http://identifiers.org/mesh/disease/name'
        mesh_drug_name    = 'http://identifiers.org/mesh/drug/name'
        mesh_disease_id   = 'http://identifiers.org/mesh/disease/id'
        
        # Disease
        doid_curie          = "doid"
        doid                = "http://identifiers.org/doid"
        pharos_disease_name = "http://pharos.nih.gov/identifier/disease/name"
        
        # DRUG
        c2b2r_drug_name   = "http://chem2bio2rdf.org/drugbank/resource/Generic_Name"

        # TARGET
        c2b2r_gene        = "http://chem2bio2rdf.org/uniprot/resource/gene"

        # PATHWAY
        c2b2r_pathway     = "http://chem2bio2rdf.org/kegg/resource/kegg_pathway"
        
        # Semantic equivalence
        
        self.equivalence = defaultdict(lambda: [])
        self.equivalence[doid_curie] = [ doid ]

        # https://github.com/prefixcommons/biocontext/blob/master/registry/uber_context.jsonld
        uber_context_path = os.path.join(os.path.dirname(__file__), 'jsonld', 'uber_context.jsonld')
        with open (uber_context_path, 'r') as stream:
            self.equivalence = json.loads (stream.read ())["@context"]
            self.equivalence["MESH"] = self.equivalence ["MESH.2013"]
            
        # Domain translation
        self.translator_router = defaultdict (lambda: defaultdict (lambda: NoTranslation ()))
        self.translator_router[mesh_disease_name][mesh_drug_name] = lambda disease: self.chemotext.disease_name_to_drug_name (disease)
        self.translator_router[doid][mesh_disease_id]             = lambda doid:    self.disease_ontology.doid_to_mesh (doid.upper())
        self.translator_router[c2b2r_drug_name][c2b2r_gene]       = lambda drug:    self.chembio_ks.drug_name_to_gene_symbol (drug)
        self.translator_router[c2b2r_gene][c2b2r_pathway]         = lambda gene:    self.chembio_ks.gene_symbol_to_pathway (gene)
        self.translator_router[c2b2r_gene][pharos_disease_name]   = lambda gene:    self.pharos.target_to_disease (gene)
        self.translator_router[mesh][root_kind]                   = lambda mesh_id: self.oxo.mesh_to_other (mesh_id)
        
    def resolve_id (self, an_id, domain):
        if not an_id in domain:
            candidate = an_id
            an_id = None
            for alternative in self.equivalence[candidate]:
                if alternative in domain:
                    # Postpone the problem of synonymy
                    an_id = alternative
                    logger.debug ("Selected alternative id {0} for input {1}".format (an_id, candidate))
                    break
                # Also, if all candidates turn out not to be in the domain, we could recurse to try synonyms for them
        return an_id
    
    def translate (self, thing, domainA, domainB):
        result = None
        resolvedA = self.resolve_id (domainA, self.translator_router)
        resolvedB = self.resolve_id (domainB, self.translator_router[domainA])
        if resolvedA and resolvedB:
            result = self.translator_router[resolvedA][resolvedB] (thing)
            if isinstance (result, NoTranslation):
                raise NoTranslation ("No translation implemented from domain {0} to domain {1}".format (domainA, domainB))
        return result
    '''
    
